@@ -46,6 +46,9 @@ Exponential <- R6::R6Class(
         if (is.null(s2)) {s2 <- self$s2}
       }
       theta <- 10^beta
+      if (self$isotropic && length(theta) == self$beta_length) {
+        theta <- rep(theta, self$D)
+      }
       if (is.null(y)) {
         if (is.matrix(x)) {
           # val <- outer(1:nrow(x), 1:nrow(x), Vectorize(function(i,j){self$kone(x[i,],x[j,],theta=theta, s2=s2)}))
@@ -107,6 +110,10 @@ Exponential <- R6::R6Class(
       # lenparams <- length(params)
       # beta <- params[1:(lenparams - 1)]
       theta <- 10^beta
+      if (self$isotropic && length(theta) == self$beta_length) {
+        theta <- rep(theta, self$D)
+      }
+
       log10 <- log(10)
       # logs2 <- params[lenparams]
       s2 <- 10 ^ logs2
@@ -119,7 +126,7 @@ Exponential <- R6::R6Class(
 
       lenparams_D <- self$beta_length*self$beta_est + self$s2_est
 
-      if (self$useC) {
+      if (self$useC && !self$isotropic) {
         dC_dparams <- kernel_exponential_dC(X, theta, C_nonug, self$s2_est,
                                             self$beta_est, lenparams_D,
                                             s2*nug, s2)
@@ -134,13 +141,20 @@ Exponential <- R6::R6Class(
           for (i in seq(1, n-1, 1)) {
             for (j in seq(i+1, n, 1)) {
               t1 <- -1 * C_nonug[i,j] * log10 * .5 / (-log(C[i,j]/s2))
-              for (k in 1:length(beta)) {
-                if (X[i,k] == X[j,k]) {
-                  dC_dparams[k,i,j] <- 0
-                } else {
-                  dC_dparams[k,i,j] <- t1 * (X[i,k] - X[j,k])^2 * theta[k]
+              if (!self$isotropic) {
+                for (k in 1:length(beta)) {
+                  if (X[i,k] == X[j,k]) {
+                    dC_dparams[k,i,j] <- 0
+                  } else {
+                    dC_dparams[k,i,j] <- t1 * (X[i,k] - X[j,k])^2 * theta[k]
+                  }
+                  dC_dparams[k,j,i] <- dC_dparams[k,i,j]
                 }
-                dC_dparams[k,j,i] <- dC_dparams[k,i,j]
+              } else {
+                # dC_dparams[1,i,j] <- -C_nonug[i,j] * .5 * t1 * log10
+                # dC_dparams[1,i,j] <- -s2 * exp(-t1) * .5 * t1 * log10
+                dC_dparams[1,i,j] <- t1 * sum((X[i,] - X[j,])^2) * theta[1]
+                dC_dparams[1,j,i] <- dC_dparams[1,i,j]
               }
             }
           }
@@ -161,6 +175,9 @@ Exponential <- R6::R6Class(
     #' @param s2 Variance parameter
     dC_dx = function(XX, X, theta, beta=self$beta, s2=self$s2) {
       if (missing(theta)) {theta <- 10^beta}
+      if (self$isotropic && length(theta) == self$beta_length) {
+        theta <- rep(theta, self$D)
+      }
       if (!is.matrix(XX)) {stop()}
       d <- ncol(XX)
       if (ncol(X) != d) {stop()}
@@ -199,10 +216,13 @@ Exponential <- R6::R6Class(
 #' @param s2_upper Upper bound for s2
 #' @param s2_est Should s2 be estimated?
 #' @param useC Should C code used? Much faster.
+#' @param isotropic If isotropic then a single beta/theta is used for all
+#' dimensions. If not (anisotropic) then a separate beta/beta is used for
+#' each dimension.
 k_Exponential <- function(beta, s2=1, D,
                           beta_lower=-8, beta_upper=6, beta_est=TRUE,
                           s2_lower=1e-8, s2_upper=1e8, s2_est=TRUE,
-                          useC=TRUE) {
+                          useC=TRUE, isotropic=FALSE) {
   Exponential$new(
     beta=beta,
     s2=s2,
@@ -213,6 +233,7 @@ k_Exponential <- function(beta, s2=1, D,
     s2_lower=s2_lower,
     s2_upper=s2_upper,
     s2_est=s2_est,
-    useC=useC
+    useC=useC,
+    isotropic=isotropic
   )
 }
