@@ -63,6 +63,9 @@ Matern52 <- R6::R6Class(
         if (is.null(s2)) {s2 <- self$s2}
       }
       theta <- 10^beta
+      if (self$isotropic && length(theta) == self$beta_length) {
+        theta <- rep(theta, self$D)
+      }
       if (is.null(y)) {
         if (is.matrix(x)) {
           # val <- outer(1:nrow(x), 1:nrow(x), Vectorize(function(i,j){self$kone(x[i,],x[j,],theta=theta, s2=s2)}))
@@ -129,6 +132,10 @@ Matern52 <- R6::R6Class(
       # lenparams <- length(params)
       # beta <- params[1:(lenparams - 1)]
       theta <- 10^beta
+      if (self$isotropic && length(theta) == self$beta_length) {
+        theta <- rep(theta, self$D)
+      }
+
       log10 <- log(10)
       # logs2 <- params[lenparams]
       s2 <- 10 ^ logs2
@@ -141,7 +148,7 @@ Matern52 <- R6::R6Class(
 
       lenparams_D <- self$beta_length*self$beta_est + self$s2_est
 
-      if (self$useC) {
+      if (self$useC && !self$isotropic) {
         dC_dparams <- kernel_matern52_dC(X, theta, C_nonug, self$s2_est,
                                          self$beta_est, lenparams_D, s2*nug)
       } else {
@@ -157,15 +164,24 @@ Matern52 <- R6::R6Class(
               tx2 <- sum(theta * (X[i,]-X[j,])^2)
               if (tx2 == 0) { # Avoid divide by 0 error
                 # When x are equal, changing param has no effect on correlation
-                dC_dparams[1:length(beta),i,j] <- dC_dparams[1:length(beta),j,i] <- 0
+                dC_dparams[1:self$beta_length,i,j] <-
+                  dC_dparams[1:self$beta_length,j,i] <- 0
               } else {
                 t1 <- sqrt(5 * tx2)
-                t3 <- C[i,j] * ((1+2*t1/3)/(1+t1+t1^2/3) - 1) * self$sqrt5 * log10
-                half_over_sqrttx2 <- .5 / sqrt(tx2)
-                for (k in 1:length(beta)) {
-                  dt1dbk <- half_over_sqrttx2 * (X[i,k] - X[j,k])^2
-                  dC_dparams[k,i,j] <- t3 * dt1dbk * theta[k]
-                  dC_dparams[k,j,i] <- dC_dparams[k,i,j]
+                if (!self$isotropic) {
+                  t3 <- C[i,j] * ((1+2*t1/3)/(1+t1+t1^2/3) - 1) * self$sqrt5 * log10
+                  half_over_sqrttx2 <- .5 / sqrt(tx2)
+                  for (k in 1:length(beta)) {
+                    dt1dbk <- half_over_sqrttx2 * (X[i,k] - X[j,k])^2
+                    dC_dparams[k,i,j] <- t3 * dt1dbk * theta[k]
+                    dC_dparams[k,j,i] <- dC_dparams[k,i,j]
+                  }
+                } else {
+
+                  dC_dparams[1,i,j] <- (
+                    s2 * t1 * log10 / 2 * exp(-t1) * (-t1/3 - t1^2/3)
+                  )
+                  dC_dparams[1,j,i] <- dC_dparams[1,i,j]
                 }
               }
             }
@@ -188,6 +204,9 @@ Matern52 <- R6::R6Class(
     #' @param s2 Variance parameter
     dC_dx = function(XX, X, theta, beta=self$beta, s2=self$s2) {
       if (missing(theta)) {theta <- 10^beta}
+      if (self$isotropic && length(theta) == self$beta_length) {
+        theta <- rep(theta, self$D)
+      }
       if (!is.matrix(XX)) {stop()}
       d <- ncol(XX)
       if (ncol(X) != d) {stop()}
@@ -195,12 +214,12 @@ Matern52 <- R6::R6Class(
       nn <- nrow(XX)
       dC_dx <- array(NA, dim=c(nn, d, n))
       for (i in 1:nn) {
-        for (k in 1:n) {
-          r <- sqrt(sum(theta * (XX[i,] - X[k,]) ^ 2))
-          for (j in 1:d) {
-            dC_dx[i, j, k] <- (
+        for (j in 1:n) {
+          r <- sqrt(sum(theta * (XX[i,] - X[j,]) ^ 2))
+          for (k in 1:d) {
+            dC_dx[i, k, j] <- (
               (-5/3 - 5/3*self$sqrt5*r) * s2 * exp(-self$sqrt5 * r) *
-                theta[j] * (XX[i, j] - X[k, j])
+                theta[k] * (XX[i, k] - X[j, k])
             )
           }
         }
@@ -229,10 +248,13 @@ Matern52 <- R6::R6Class(
 #' @param s2_upper Upper bound for s2
 #' @param s2_est Should s2 be estimated?
 #' @param useC Should C code used? Much faster.
+#' @param isotropic If isotropic then a single beta/theta is used for all
+#' dimensions. If not (anisotropic) then a separate beta/beta is used for
+#' each dimension.
 k_Matern52 <- function(beta, s2=1, D,
                        beta_lower=-8, beta_upper=6, beta_est=TRUE,
                        s2_lower=1e-8, s2_upper=1e8, s2_est=TRUE,
-                       useC=TRUE) {
+                       useC=TRUE, isotropic=FALSE) {
   Matern52$new(
     beta=beta,
     s2=s2,
@@ -243,6 +265,7 @@ k_Matern52 <- function(beta, s2=1, D,
     s2_lower=s2_lower,
     s2_upper=s2_upper,
     s2_est=s2_est,
-    useC=useC
+    useC=useC,
+    isotropic=isotropic
   )
 }
